@@ -18,7 +18,6 @@
  */
 
 #include "gsd-wacom-oled-constants.h"
-#include "gsd-device-manager.h"
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
@@ -27,7 +26,7 @@
 
 G_BEGIN_DECLS
 
-gboolean set_oled (GsdDevice *device, guint button, char *label, GError **error);
+gboolean set_oled (const gchar *device_path, gboolean left_handed, guint button, char *label, GError **error);
 char *gsd_wacom_oled_gdkpixbuf_to_base64 (GdkPixbuf *pixbuf);
 
 G_END_DECLS
