@@ -33,6 +33,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gdesktop-enums.h>
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
@@ -42,18 +43,22 @@
 #include "gsd-xsettings-manager.h"
 #include "gsd-xsettings-gtk.h"
 #include "gnome-settings-bus.h"
+#include "gsd-settings-migrate.h"
 #include "xsettings-manager.h"
 #include "fc-monitor.h"
 #include "gsd-remote-display-manager.h"
 #include "wm-button-layout-translation.h"
 
-#define MOUSE_SETTINGS_SCHEMA     "org.gnome.settings-daemon.peripherals.mouse"
+#define MOUSE_SETTINGS_SCHEMA     "org.gnome.desktop.peripherals.mouse"
 #define BACKGROUND_SETTINGS_SCHEMA "org.gnome.desktop.background"
 #define INTERFACE_SETTINGS_SCHEMA "org.gnome.desktop.interface"
 #define SOUND_SETTINGS_SCHEMA     "org.gnome.desktop.sound"
 #define PRIVACY_SETTINGS_SCHEMA     "org.gnome.desktop.privacy"
 #define WM_SETTINGS_SCHEMA        "org.gnome.desktop.wm.preferences"
 #define A11Y_SCHEMA               "org.gnome.desktop.a11y"
+#define A11Y_INTERFACE_SCHEMA     "org.gnome.desktop.a11y.interface"
+#define A11Y_APPLICATIONS_SCHEMA   "org.gnome.desktop.a11y.applications"
+#define INPUT_SOURCES_SCHEMA       "org.gnome.desktop.input-sources"
 #define CLASSIC_WM_SETTINGS_SCHEMA "org.gnome.shell.extensions.classic-overrides"
 
 #define XSETTINGS_PLUGIN_SCHEMA "org.gnome.settings-daemon.plugins.xsettings"
@@ -66,13 +71,24 @@
 #define CURSOR_SIZE_KEY "cursor-size"
 #define CURSOR_THEME_KEY "cursor-theme"
 
-#define FONT_ANTIALIASING_KEY "antialiasing"
-#define FONT_HINTING_KEY      "hinting"
-#define FONT_RGBA_ORDER_KEY   "rgba-order"
+#define FONT_ANTIALIASING_KEY "font-antialiasing"
+#define FONT_HINTING_KEY      "font-hinting"
+#define FONT_RGBA_ORDER_KEY   "font-rgba-order"
+
+#define HIGH_CONTRAST_KEY "high-contrast"
+
+#define INPUT_SOURCES_KEY      "sources"
+#define OSK_ENABLED_KEY        "screen-keyboard-enabled"
+#define GTK_IM_MODULE_KEY      "gtk-im-module"
 
 #define GTK_SETTINGS_DBUS_PATH "/org/gtk/Settings"
 #define GTK_SETTINGS_DBUS_NAME "org.gtk.Settings"
 
+#define INPUT_SOURCE_TYPE_IBUS "ibus"
+
+#define GTK_IM_MODULE_SIMPLE "gtk-im-context-simple"
+#define GTK_IM_MODULE_IBUS   "ibus"
+
 static const gchar introspection_xml[] =
 "<node name='/org/gtk/Settings'>"
 "  <interface name='org.gtk.Settings'>"
@@ -275,6 +291,11 @@ struct _GsdXSettingsManager
         FcMonitor         *fontconfig_monitor;
         gint64             fontconfig_timestamp;
 
+        GSettings         *interface_settings;
+        GSettings         *input_sources_settings;
+        GSettings         *a11y_settings;
+        GdkSeat           *user_seat;
+
         GsdXSettingsGtk   *gtk;
 
         guint              introspect_properties_changed_id;
@@ -284,6 +305,9 @@ struct _GsdXSettingsManager
         guint              display_config_watch_id;
         guint              monitors_changed_id;
 
+        guint              device_added_id;
+        guint              device_removed_id;
+
         guint              shell_name_watch_id;
         gboolean           have_shell;
 
@@ -377,6 +401,25 @@ translate_button_layout (GsdXSettingsManager *manager,
         g_free (layout);
 }
 
+static void
+translate_theme_name (GsdXSettingsManager *manager,
+                      TranslationEntry    *trans,
+                      GVariant            *value)
+{
+        GSettings *settings;
+        gboolean hc = FALSE;
+
+        settings = g_hash_table_lookup (manager->settings, A11Y_INTERFACE_SCHEMA);
+
+        if (settings)
+                hc = g_settings_get_boolean (settings, HIGH_CONTRAST_KEY);
+
+        xsettings_manager_set_string (manager->manager,
+                                      trans->xsetting_name,
+                                      hc ? "HighContrast"
+                                         : g_variant_get_string (value, NULL));
+}
+
 static void
 fixed_false_int (GsdXSettingsManager *manager,
                  FixedEntry          *fixed)
@@ -465,8 +508,8 @@ static FixedEntry fixed_entries [] = {
 };
 
 static TranslationEntry translations [] = {
-        { "org.gnome.settings-daemon.peripherals.mouse", "double-click",   "Net/DoubleClickTime",  translate_int_int },
-        { "org.gnome.settings-daemon.peripherals.mouse", "drag-threshold", "Net/DndDragThreshold", translate_int_int },
+        { "org.gnome.desktop.peripherals.mouse", "double-click",   "Net/DoubleClickTime",  translate_int_int },
+        { "org.gnome.desktop.peripherals.mouse", "drag-threshold", "Net/DndDragThreshold", translate_int_int },
 
         { "org.gnome.desktop.background", "show-desktop-icons",    "Gtk/ShellShowsDesktop",   translate_bool_int },
 
@@ -475,9 +518,9 @@ static TranslationEntry translations [] = {
         { "org.gnome.desktop.interface", "cursor-blink",           "Net/CursorBlink",         translate_bool_int },
         { "org.gnome.desktop.interface", "cursor-blink-time",      "Net/CursorBlinkTime",     translate_int_int },
         { "org.gnome.desktop.interface", "cursor-blink-timeout",   "Gtk/CursorBlinkTimeout",  translate_int_int },
-        { "org.gnome.desktop.interface", "gtk-theme",              "Net/ThemeName",           translate_string_string },
+        { "org.gnome.desktop.interface", "gtk-theme",              "Net/ThemeName",           translate_theme_name },
         { "org.gnome.desktop.interface", "gtk-im-module",          "Gtk/IMModule",            translate_string_string },
-        { "org.gnome.desktop.interface", "icon-theme",             "Net/IconThemeName",       translate_string_string },
+        { "org.gnome.desktop.interface", "icon-theme",             "Net/IconThemeName",       translate_theme_name },
         { "org.gnome.desktop.interface", "cursor-theme",           "Gtk/CursorThemeName",     translate_string_string },
         { "org.gnome.desktop.interface", "gtk-enable-primary-paste", "Gtk/EnablePrimaryPaste", translate_bool_int },
         { "org.gnome.desktop.interface", "overlay-scrolling",      "Gtk/OverlayScrolling",    translate_bool_int },
@@ -670,21 +713,21 @@ xft_settings_get (GsdXSettingsManager *manager,
                   GsdXftSettings    *settings)
 {
 	GSettings  *interface_settings;
-        GsdFontAntialiasingMode antialiasing;
-        GsdFontHinting hinting;
-        GsdFontRgbaOrder order;
+        GDesktopFontAntialiasingMode antialiasing;
+        GDesktopFontHinting hinting;
+        GDesktopFontRgbaOrder order;
         gboolean use_rgba = FALSE;
         double dpi;
         int cursor_size;
 
 	interface_settings = g_hash_table_lookup (manager->settings, INTERFACE_SETTINGS_SCHEMA);
 
-        antialiasing = g_settings_get_enum (manager->plugin_settings, FONT_ANTIALIASING_KEY);
-        hinting = g_settings_get_enum (manager->plugin_settings, FONT_HINTING_KEY);
-        order = g_settings_get_enum (manager->plugin_settings, FONT_RGBA_ORDER_KEY);
+        antialiasing = g_settings_get_enum (interface_settings, FONT_ANTIALIASING_KEY);
+        hinting = g_settings_get_enum (interface_settings, FONT_HINTING_KEY);
+        order = g_settings_get_enum (interface_settings, FONT_RGBA_ORDER_KEY);
 
-        settings->antialias = (antialiasing != GSD_FONT_ANTIALIASING_MODE_NONE);
-        settings->hinting = (hinting != GSD_FONT_HINTING_NONE);
+        settings->antialias = (antialiasing != G_DESKTOP_FONT_ANTIALIASING_MODE_NONE);
+        settings->hinting = (hinting != G_DESKTOP_FONT_HINTING_NONE);
         settings->window_scale = get_window_scale (manager);
         dpi = get_dpi_from_gsettings (manager);
         settings->dpi = dpi * 1024; /* Xft wants 1/1024ths of an inch */
@@ -696,46 +739,46 @@ xft_settings_get (GsdXSettingsManager *manager,
         settings->hintstyle = "hintfull";
 
         switch (hinting) {
-        case GSD_FONT_HINTING_NONE:
+        case G_DESKTOP_FONT_HINTING_NONE:
                 settings->hintstyle = "hintnone";
                 break;
-        case GSD_FONT_HINTING_SLIGHT:
+        case G_DESKTOP_FONT_HINTING_SLIGHT:
                 settings->hintstyle = "hintslight";
                 break;
-        case GSD_FONT_HINTING_MEDIUM:
+        case G_DESKTOP_FONT_HINTING_MEDIUM:
                 settings->hintstyle = "hintmedium";
                 break;
-        case GSD_FONT_HINTING_FULL:
+        case G_DESKTOP_FONT_HINTING_FULL:
                 settings->hintstyle = "hintfull";
                 break;
         }
 
         switch (order) {
-        case GSD_FONT_RGBA_ORDER_RGBA:
+        case G_DESKTOP_FONT_RGBA_ORDER_RGBA:
                 settings->rgba = "rgba";
                 break;
-        case GSD_FONT_RGBA_ORDER_RGB:
+        case G_DESKTOP_FONT_RGBA_ORDER_RGB:
                 settings->rgba = "rgb";
                 break;
-        case GSD_FONT_RGBA_ORDER_BGR:
+        case G_DESKTOP_FONT_RGBA_ORDER_BGR:
                 settings->rgba = "bgr";
                 break;
-        case GSD_FONT_RGBA_ORDER_VRGB:
+        case G_DESKTOP_FONT_RGBA_ORDER_VRGB:
                 settings->rgba = "vrgb";
                 break;
-        case GSD_FONT_RGBA_ORDER_VBGR:
+        case G_DESKTOP_FONT_RGBA_ORDER_VBGR:
                 settings->rgba = "vbgr";
                 break;
         }
 
         switch (antialiasing) {
-        case GSD_FONT_ANTIALIASING_MODE_NONE:
+        case G_DESKTOP_FONT_ANTIALIASING_MODE_NONE:
                 settings->antialias = 0;
                 break;
-        case GSD_FONT_ANTIALIASING_MODE_GRAYSCALE:
+        case G_DESKTOP_FONT_ANTIALIASING_MODE_GRAYSCALE:
                 settings->antialias = 1;
                 break;
-        case GSD_FONT_ANTIALIASING_MODE_RGBA:
+        case G_DESKTOP_FONT_ANTIALIASING_MODE_RGBA:
                 settings->antialias = 1;
                 use_rgba = TRUE;
         }
@@ -896,8 +939,6 @@ plugin_callback (GSettings           *settings,
                 /* Do nothing, as GsdXsettingsGtk will handle it */
         } else if (g_str_equal (key, XSETTINGS_OVERRIDE_KEY)) {
                 override_callback (settings, key, manager);
-        } else {
-                xft_callback (settings, key, manager);
         }
 }
 
@@ -1010,12 +1051,25 @@ xsettings_callback (GSettings           *settings,
         GVariant         *value;
 
         if (g_str_equal (key, TEXT_SCALING_FACTOR_KEY) ||
+            g_str_equal (key, FONT_ANTIALIASING_KEY) ||
+            g_str_equal (key, FONT_HINTING_KEY) ||
+            g_str_equal (key, FONT_RGBA_ORDER_KEY) ||
             g_str_equal (key, CURSOR_SIZE_KEY) ||
             g_str_equal (key, CURSOR_THEME_KEY)) {
         	xft_callback (NULL, key, manager);
         	return;
 	}
 
+        if (g_str_equal (key, HIGH_CONTRAST_KEY)) {
+                GSettings *iface_settings;
+
+                iface_settings = g_hash_table_lookup (manager->settings,
+                                                      INTERFACE_SETTINGS_SCHEMA);
+                xsettings_callback (iface_settings, "gtk-theme", manager);
+                xsettings_callback (iface_settings, "icon-theme", manager);
+                return;
+        }
+
         trans = find_translation_entry (settings, key);
         if (trans == NULL) {
                 return;
@@ -1262,6 +1316,138 @@ launch_xwayland_services (void)
         }
 }
 
+static void
+migrate_settings (void)
+{
+        GsdSettingsMigrateEntry xsettings_entries[] = {
+                { "antialiasing", "font-antialiasing", NULL },
+                { "hinting", "font-hinting", NULL },
+                { "rgba-order", "font-rgba-order", NULL },
+        };
+        GsdSettingsMigrateEntry mouse_entries[] = {
+                { "double-click", "double-click", NULL },
+                { "drag-threshold", "drag-threshold", NULL },
+        };
+
+        gsd_settings_migrate_check ("org.gnome.settings-daemon.plugins.xsettings.deprecated",
+                                    "/org/gnome/settings-daemon/plugins/xsettings/",
+                                    "org.gnome.desktop.interface",
+                                    "/org/gnome/desktop/interface/",
+                                    xsettings_entries, G_N_ELEMENTS (xsettings_entries));
+
+        gsd_settings_migrate_check ("org.gnome.settings-daemon.peripherals.mouse.deprecated",
+                                    "/org/gnome/settings-daemon/peripherals/mouse/",
+                                    "org.gnome.desktop.peripherals.mouse",
+                                    "/org/gnome/desktop/peripherals/mouse/",
+                                    mouse_entries, G_N_ELEMENTS (mouse_entries));
+}
+
+static gboolean
+need_ibus (GsdXSettingsManager *manager)
+{
+        GVariant *sources;
+        GVariantIter iter;
+        const gchar *type;
+        gboolean needs_ibus = FALSE;
+
+        sources = g_settings_get_value (manager->input_sources_settings,
+                                        INPUT_SOURCES_KEY);
+
+        g_variant_iter_init (&iter, sources);
+        while (g_variant_iter_next (&iter, "(&s&s)", &type, NULL)) {
+                if (g_str_equal (type, INPUT_SOURCE_TYPE_IBUS)) {
+                        needs_ibus = TRUE;
+                        break;
+                }
+        }
+
+        g_variant_unref (sources);
+
+        return needs_ibus;
+}
+
+static gboolean
+need_osk (GsdXSettingsManager *manager)
+{
+        gboolean has_touchscreen = FALSE;
+        GList *devices;
+        GdkSeat *seat;
+
+        if (g_settings_get_boolean (manager->a11y_settings,
+                                    OSK_ENABLED_KEY))
+                return TRUE;
+
+        seat = gdk_display_get_default_seat (gdk_display_get_default ());
+        devices = gdk_seat_get_slaves (seat, GDK_SEAT_CAPABILITY_TOUCH);
+
+        has_touchscreen = devices != NULL;
+
+        g_list_free (devices);
+
+        return has_touchscreen;
+}
+
+static void
+update_gtk_im_module (GsdXSettingsManager *manager)
+{
+        const gchar *module;
+        gchar *setting;
+
+        setting = g_settings_get_string (manager->interface_settings,
+                                         GTK_IM_MODULE_KEY);
+        if (setting && *setting)
+                module = setting;
+        else if (need_ibus (manager) || need_osk (manager))
+                module = GTK_IM_MODULE_IBUS;
+        else
+                module = GTK_IM_MODULE_SIMPLE;
+
+        xsettings_manager_set_string (manager->manager, "Gtk/IMModule", module);
+        g_free (setting);
+}
+
+static void
+device_added_cb (GdkSeat             *user_seat,
+                 GdkDevice           *device,
+                 GsdXSettingsManager *manager)
+{
+        GdkInputSource source;
+
+        source = gdk_device_get_source (device);
+        if (source == GDK_SOURCE_TOUCHSCREEN) {
+                update_gtk_im_module (manager);
+        }
+}
+
+static void
+device_removed_cb (GdkSeat             *user_seat,
+                   GdkDevice           *device,
+                   GsdXSettingsManager *manager)
+{
+        GdkInputSource source;
+
+        source = gdk_device_get_source (device);
+        if (source == GDK_SOURCE_TOUCHSCREEN)
+                update_gtk_im_module (manager);
+}
+
+static void
+set_devicepresence_handler (GsdXSettingsManager *manager)
+{
+        GdkSeat *user_seat;
+
+        if (gnome_settings_is_wayland ())
+                return;
+
+        user_seat = gdk_display_get_default_seat (gdk_display_get_default ());
+
+        manager->device_added_id = g_signal_connect (G_OBJECT (user_seat), "device-added",
+                                                     G_CALLBACK (device_added_cb), manager);
+        manager->device_removed_id = g_signal_connect (G_OBJECT (user_seat), "device-removed",
+                                                       G_CALLBACK (device_removed_cb), manager);
+        manager->user_seat = user_seat;
+}
+
 gboolean
 gsd_xsettings_manager_start (GsdXSettingsManager *manager,
                              GError             **error)
@@ -1274,6 +1460,8 @@ gsd_xsettings_manager_start (GsdXSettingsManager *manager,
         g_debug ("Starting xsettings manager");
         gnome_settings_profile_start (NULL);
 
+        migrate_settings ();
+
         if (!setup_xsettings_managers (manager)) {
                 g_set_error (error, GSD_XSETTINGS_ERROR,
                              GSD_XSETTINGS_ERROR_INIT,
@@ -1281,6 +1469,23 @@ gsd_xsettings_manager_start (GsdXSettingsManager *manager,
                 return FALSE;
         }
 
+	set_devicepresence_handler (manager);
+        manager->interface_settings = g_settings_new (INTERFACE_SETTINGS_SCHEMA);
+        g_signal_connect_swapped (manager->interface_settings,
+                                  "changed::" GTK_IM_MODULE_KEY,
+                                  G_CALLBACK (update_gtk_im_module), manager);
+
+        manager->input_sources_settings = g_settings_new (INPUT_SOURCES_SCHEMA);
+        g_signal_connect_swapped (manager->input_sources_settings,
+                                  "changed::" INPUT_SOURCES_KEY,
+                                  G_CALLBACK (update_gtk_im_module), manager);
+
+        manager->a11y_settings = g_settings_new (A11Y_APPLICATIONS_SCHEMA);
+        g_signal_connect_swapped (manager->a11y_settings,
+                                  "changed::" OSK_ENABLED_KEY,
+                                  G_CALLBACK (update_gtk_im_module), manager);
+        update_gtk_im_module (manager);
+
         manager->monitors_changed_id =
                 g_dbus_connection_signal_subscribe (manager->dbus_connection,
                                                     "org.gnome.Mutter.DisplayConfig",
@@ -1338,6 +1543,8 @@ gsd_xsettings_manager_start (GsdXSettingsManager *manager,
                              WM_SETTINGS_SCHEMA, g_settings_new (WM_SETTINGS_SCHEMA));
         g_hash_table_insert (manager->settings,
                              A11Y_SCHEMA, g_settings_new (A11Y_SCHEMA));
+        g_hash_table_insert (manager->settings,
+                             A11Y_INTERFACE_SCHEMA, g_settings_new (A11Y_INTERFACE_SCHEMA));
 
         session = g_getenv ("XDG_CURRENT_DESKTOP");
         if (session && strstr (session, "GNOME-Classic")) {
@@ -1476,6 +1683,16 @@ gsd_xsettings_manager_stop (GsdXSettingsManager *manager)
                 g_object_unref (manager->gtk);
                 manager->gtk = NULL;
         }
+
+        if (manager->user_seat != NULL) {
+                g_signal_handler_disconnect (manager->user_seat, manager->device_added_id);
+                g_signal_handler_disconnect (manager->user_seat, manager->device_removed_id);
+                manager->user_seat = NULL;
+        }
+
+        g_clear_object (&manager->a11y_settings);
+        g_clear_object (&manager->input_sources_settings);
+        g_clear_object (&manager->interface_settings);
 }
 
 static void
